@@ -20,7 +20,7 @@ class SubprocessTests(object):
     def start_subprocess(self, cmd):
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.proc = subprocess.Popen(cmd)
-        time.sleep(.2)
+        time.sleep(.5)
         if self.proc.returncode is not None: # pragma: no cover
             raise RuntimeError('%s didnt start' % str(cmd))
 
@@ -87,6 +87,19 @@ class EchoTests(SubprocessTests, unittest.TestCase):
         self.assertEqual(headers.get('server'), 'waitress')
         self.assertTrue(headers.get('date'))
 
+    def test_bad_host_header(self):
+        # http://corte.si/posts/code/pathod/pythonservers/index.html
+        to_send = ("GET / HTTP/1.0\n"
+                   " Host: 0\n\n")
+        to_send = tobytes(to_send)
+        self.sock.connect((self.host, self.port))
+        self.sock.send(to_send)
+        fp = self.sock.makefile('rb', 0)
+        line, headers, response_body = read_http(fp)
+        self.assertline(line, '400', 'Bad Request', 'HTTP/1.0')
+        self.assertEqual(headers.get('server'), 'waitress')
+        self.assertTrue(headers.get('date'))
+
     def test_send_with_body(self):
         to_send = ("GET / HTTP/1.0\n"
                    "Content-Length: 5\n\n")
@@ -1227,7 +1240,14 @@ def read_http(fp): # pragma: no cover
   
     if 'content-length' in headers:
         num = int(headers['content-length'])
-        body = fp.read(num)
+        body = b''
+        left = num
+        while left > 0:
+            data = fp.read(left)
+            if not data:
+                break
+            body += data
+            left -= len(data)
     else:
         # read until EOF
         body = fp.read()
